/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2023 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#ifndef OSM_TAGS_H
#define OSM_TAGS_H

#include <genht/htsp.h>
typedef htsp_t osm_tags_t;

void osm_tags_init(osm_tags_t *tags);
void osm_tags_uninit(osm_tags_t *tags);
void osm_tags_copy(osm_tags_t *dst, const osm_tags_t *src);
void osm_tags_dup(osm_tags_t *dst, const osm_tags_t *src);
void osm_tag_set(osm_tags_t *dst, const char *key, const char *val);

/* Returns 1 if a and b have the same data, 0 otherwise */
int osm_tags_eq(const osm_tags_t *a, const osm_tags_t *b);

/* Returns 1 if a and b can not be merged because of different value for
   the same key */
int osm_tags_conflict(const osm_tags_t *a, const osm_tags_t *b);

/* Copy all tags from src to dst (until the first conflict). Return 0
   on error, -1 on conflict. Conflict happens only if a tag key is
   present both in src and dst but with different values. */
int osm_tags_merge(osm_tags_t *dst, const osm_tags_t *src);

#endif

