/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer - generic tile cache
 *  Copyright (C) 2023,2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#ifndef OSM_LIB_TILECACHE_H
#define OSM_LIB_TILECACHE_H

#include <time.h>
#include <librnd/hid/pixmap.h>

typedef struct osm_tile_s {
	char *tilekey; /* also the hash key */
	rnd_pixmap_t *pm;
	double mtime;
	time_t last_drawn;
} osm_tile_t;

typedef struct osm_tilecache_s osm_tilecache_t;

struct osm_tilecache_s {
	/*** fields managed by the caller ***/
	long inited;
	unsigned offline:1;  /* do not yet download (e.g. during API init) */

	const char *(*build_url_for)(rnd_design_t *dsg, osm_tilecache_t *ctx, const char *tilekey, void *tileaddr);
	void (*snprint_disk_name)(rnd_design_t *dsg, osm_tilecache_t *ctx, const char *tilekey, void *tileaddr, char *dst, size_t max_len);

	void *user_data;

	/*** fields used internally ***/
	htsp_t cache;              /* key: tilekey; val: (osm_tile_t *) */
	char *cache_disk_path;     /* allocated with 256 extra characters at the end */
	char *cache_disk_path_end;
};

void osm_tilecache_init(osm_tilecache_t *ctx);
rnd_pixmap_t *osm_tilecache_get(rnd_design_t *dsg, osm_tilecache_t *ctx, const char *tilekey, void *tileaddr, time_t now);

#endif
