/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer - standard tools
 *  Copyright (C) 2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include "config.h"

#include <librnd/hid/tool.h>

#include "data.h"
#include "search.h"
#include "crosshair.h"


void osm_tool_remove_press(rnd_design_t *hl)
{
	osm_design_t *osm = (osm_design_t *)hl;
	osm_layer_t *ly = osm_layer_get_curr(osm);
	osm_any_obj_t *obj;

	if (ly == NULL) {
		rnd_message(RND_MSG_ERROR, "Select a layer first!\n");
		return;
	}

	if (!ly->editable) {
		rnd_message(RND_MSG_ERROR, "Can't remove object: layer is not editable\n");
		return;
	}

	obj = osm_search_obj_xy(ly, osm->crosshair_x, osm->crosshair_y);

	if (obj != NULL) {
		uundo_freeze_serial(&ly->undo);

		osm_obj_remove(obj, 1);
	
		uundo_unfreeze_serial(&ly->undo);
		uundo_inc_serial(&ly->undo);
	}
}

/* XPM */
static const char *del_icon[] = {
/* columns rows colors chars-per-pixel */
"21 21 3 1",
"  c black",
". c gray100",
"X c None",
/* pixels */
"XXX XXXXXXXXXXXX XXXX",
"XXXX XXX    XXX XXXXX",
"XXXXX X      X XXXXXX",
"XXXXXX XX  XX XXXXXXX",
"XXXXX  XX  XX  XXXXXX",
"XXXXX          XXXXXX",
"XXXXXX        XXXXXXX",
"XXXXXXX  XX  XXXXXXXX",
"XXXXXXX  XX XXXXXXXXX",
"XXXXXX X XX  XXXXXXXX",
"XXXX  XX    X  XXXXXX",
"XX  XXXXX  XXXX  XXXX",
"XXXXXXXXXXXXXXXXXXXXX",
"XX     XX     X XXXXX",
"XXX XXX X XXXXX XXXXX",
"XXX XXX X XXXXX XXXXX",
"XXX XXX X XXXXX XXXXX",
"XXX XXX X    XX XXXXX",
"XXX XXX X XXXXX XXXXX",
"XXX XXX X XXXXX XXXXX",
"XX     XX     X     X"
};

rnd_tool_t osm_rnd_tool_remove = {
	"remove", NULL, NULL, 100, del_icon, RND_TOOL_CURSOR_NAMED("pirate"), 0,
	NULL,
	NULL,
	osm_tool_remove_press,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL /* escape */
};
